import { ConfigUI } from "./ConfigUI";
import { ISettings } from "./Settings";

export class CategoryFilterUI extends ConfigUI {
    $categorySelect: JQuery<HTMLSelectElement>;
    $categoryUl: JQuery;
    categoriesChanged: () => void;

    constructor(settings: ISettings, categoriesChanged: () => void) {
        super(settings);
        this.categoriesChanged = categoriesChanged;
    }

    async init() {
        var self = this;
        this.$categorySelect = $("select[name=categoryId]");
        this.$categoryUl = $(".category-row ul");

        await this.reloadTopLevelCategories();

        this.$categorySelect.change(async () => {
            var selectedCategory = self.$categorySelect.val();
            if (selectedCategory) {
                var categoryId = parseInt(selectedCategory.toString())
                await this.addTopLevelCategory(categoryId);
                self.$categoryUl.show();
                //remove selected item
                self.$categorySelect.children().filter("[value=" + categoryId + "]").remove();
                self.$categorySelect.val("0");
            }
        });
    }
    /**
     * Gets the number of pixels that the category component uses in the ui.
     */
    public getCategoryComponentHeight(): number {
        var $selectedCategories = $(".category-selected");
        if ($selectedCategories.length == 0) return 0;
        return this.$categoryUl.outerHeight(true);
    }
    public getSelectedCategories(): number[] {
        var r = [];
        var $selectedCategories = $(".category-selected");
        if ($selectedCategories && $selectedCategories.length) {
            for (var i = 0; i < $selectedCategories.length; i++) {
                var id = parseInt($($selectedCategories[i]).data("id"));
                if (id && !r.includes(id))
                    r.push(id);
            }
        }
        return r;
    }
    addOption(text: string, id: number) {
        var $o = $("<option></option>");
        $o.text(text);
        $o.val(id);
        this.$categorySelect.append($o);
    }
    async reloadTopLevelCategories() {
        var self = this;
        var sel = this.getSelectedCategories();
        //if no categories are selected, hide list of selected categories
        if (!sel || !sel.length) {
            this.$categoryUl.hide();
        }
        this.settings.dataStore.getAllCategories().then((categories) => {
            self.$categorySelect.empty();
            self.addOption(self.i18n("select-category"), 0);
            if (!categories || !categories.length) return;

            categories.forEach((ca) => {
                if (sel.includes(ca.Id)) return;
                self.addOption(ca.Name, ca.Id);
            });
        });

    }
    appendCategoryLi($ul: JQuery, cat: ImageVault.Category): void {
        var self = this;
        var children = "";
        if (cat.Categories && cat.Categories.length) {
            cat.Categories.map((c) => {
                children += "<a class='category-child category-name' data-id='" + c.Id + "' href='#'>" + ImageVault.Util.htmlEncode(c.Name) + "</a>";
            });
        }
        var $li = $("<li data-id='" + cat.Id + "'><span class='parent-category'><a class='remove-category' href='#'>x</a><span class='category-name category-selected' data-id='" + cat.Id + "'>" + ImageVault.Util.htmlEncode(cat.Name) + "</span>&gt;</span>" + children + "<ul></ul></li>")
        $ul.append($li)
        $("a.remove-category", $li).click(async (e) => { return self.removeCategory(e) });
        $("a.category-name", $li).click(async (e) => { return self.toggleCategory(e); });

    }
    async removeCategory(e: JQuery.ClickEvent) {
        e.preventDefault();
        var $a = $(e.currentTarget);
        var $li = $a.closest("li");
        var id = $li.data("id");
        //remove any active links
        $("a[data-id=" + id + "]").removeClass("category-selected");
        var $gp = $li.parent().parent();
        var isChild = ($gp.is("li"));
        $li.remove();
        if (!isChild)
            await this.reloadTopLevelCategories();
        this.notifyCategoriesChanged();
        return false;
    }
    notifyCategoriesChanged() {
        this.categoriesChanged();
    }
    async toggleCategory(e: JQuery.ClickEvent) {
        var self = this;
        e.preventDefault();
        var $a = $(e.currentTarget);
        var isSelected = $a.hasClass("category-selected");
        $a.toggleClass("category-selected");
        var id = $a.data("id");
        var $parentLi = $a.closest("li");
        if (isSelected) {
            //find child li matching id and kill it
            $("li[data-id=" + id + "]", $parentLi).remove();
        } else {
            var category = await self.settings.dataStore.getCategoryById(id);
            if (category.Categories && category.Categories.length) {
                var $ul = $("ul", $parentLi).first();
                self.appendCategoryLi($ul, category);
            }
        }
        this.notifyCategoriesChanged();
        return false;
    }
    async addTopLevelCategory(id: number) {
        var cat = await this.settings.dataStore.getCategoryById(id);
        this.logResult("INFO", "Adding category " + cat.Name);
        this.appendCategoryLi(this.$categoryUl, cat);
        this.notifyCategoriesChanged();
    }

}