import * as $ from "jquery";
import { ConfigUI, IError } from "./ConfigUI";
import { AssetInteractionUI } from "./AssetInteractionUI";
import { InfoPanelUI } from "./InfoPanelUI";
import { DataStore } from "./DataStore";
import { ISettings, IInstanceConfig } from "./Settings";
import { SearchPanelUI } from "./SearchPanelUI";
import { MediaServiceUtilUI } from "./MediaServiceUtilUI";


export interface IInstanceBroker {
    settings: ISettings;
    mediaServiceUtil: MediaServiceUtilUI;
    assetInteractionUI: AssetInteractionUI;
    selectAsset(asset: ImageVault.IAsset): Promise<void>
    infoPanel: InfoPanelUI;
}
export class InstanceConfigUI extends ConfigUI implements IInstanceBroker {

    public static readonly AssetConfigKey: string = "assetConfig";

    $instanceSettings: JQuery;

    assetInteractionUI: AssetInteractionUI;
    searchPanel: SearchPanelUI;
    infoPanel: InfoPanelUI;
    mediaServiceUtil: MediaServiceUtilUI;

    constructor(settings: ISettings) {
        super(settings);
    }
    public async init(values: any) {
        let self = this;
        try {
            var props = {
                ivUrl: this.settings.config.ivUrl,
                activationCallback: function (activationResponse: ImageVault.ActivationResponse) {
                    switch (activationResponse.state) {
                        case ImageVault.ActivationResponseState.completed:
                            self.hideActivationMessage();
                            self.alert(activationResponse.htmlMessage, "info");
                            break;
                        case ImageVault.ActivationResponseState.tentative:
                            self.showActivationMessage(activationResponse.htmlMessage);
                            break;
                        default:
                            self.showActivationError(activationResponse.htmlMessage);
                            break;
                    }
                },
                clientIdentityAuth: ConfigUI.ClientIdentityAuth,
                i18n: {
                    pin_code_request_no_valid_pin_code: self.settings.i18n.pin_client_pin_code_request_no_valid_pin_code,
                    poll_pin_failed_async_is_not_supported: self.settings.i18n.pin_client_poll_pin_failed_async_is_not_supported,
                    poll_pin_code_returned_state_x: self.settings.i18n.pin_client_poll_pin_code_returned_state_x,
                    no_access_code_was_returned_from_poll_code_response: self.settings.i18n.pin_client_no_access_code_was_returned_from_poll_code_response,
                    activation_completed: self.settings.i18n.pin_client_activation_completed,
                    activation_is_required_go_to_x_to_activate: self.settings.i18n.pin_client_activation_is_required_go_to_x_to_activate,
                }
            }
            this.settings.client = new ImageVault.PinClient(props);
            this.settings.dataStore = new DataStore(this.settings.client);

            this.settings.instanceConfig = this.getInstanceConfig(values);

            this.$instanceSettings = $("#instance-settings");

            this.mediaServiceUtil = new MediaServiceUtilUI(this.settings);
            this.infoPanel = new InfoPanelUI(this);
            this.assetInteractionUI = new AssetInteractionUI(this.settings);
            this.searchPanel = new SearchPanelUI(this);

            await Promise.all([
                this.mediaServiceUtil.init(),
                this.infoPanel.init(),
                this.assetInteractionUI.init(),
                this.searchPanel.init()
            ]);

            this.$instanceSettings.show();

        } catch (e) {
            this.error(this.i18n("error-during-initialization"), e);
        }
        this.hideLoader();

        window.validate = () => {
            try {
                //TODO: loader won't show since we do synchronous ajax calls to support the SiteVision validate method that is synchronous
                this.showLoader();
                var r = self.saveSelection();
                if (!r) return true;
                self.error(self.i18n("error-during-validate"), r);
                return false;
            } finally {
                self.hideLoader();
            }
        };
        //set Ok function
        window.getValues = function () {
            var values = window._getValues();
            //remove all fields whose values are not needed to be persisted (many of these values are bundled in the asset value)
            var fieldsToRemove = ["imageVaultUrl",//global settings
                "search", "categoryId", "searchOrderBy",//search fields
                "formatAspectRatio", "formatId", "formatHeight", "formatWidth",//format
                "alt-text-from-metadata-id-select", "alt-text-from-metadata-text", "alt-text-custom", "alt-text-choice",//alt-text metadataTextUI
                "caption-from-metadata-id-select", "caption-from-metadata-text", "caption-custom", "caption-choice",//caption MetadataTextUI
                "clickInteraction", "clickInteractionCustomUrlData"//asset interaction
            ];
            for (var i = 0; i < fieldsToRemove.length; i++) {
                var k = fieldsToRemove[i];
                if (values.hasOwnProperty(k))
                    delete values[k];
            }
            return values;
        };

    }

    getInstanceConfig(values: any): IInstanceConfig {
        var ac = this.getValue(InstanceConfigUI.AssetConfigKey);
        var settings: IInstanceConfig = {};
        if (ac) {
            settings = JSON.parse(ac);
        }
        //fallback to copy old values (this can be removed when no client uses the old code)
        if (values.hasOwnProperty("assetId") && !settings.id) {
            settings.id = values["assetId"];
        }
        if (values.hasOwnProperty("assetHtml") && !settings.html) {
            settings.html = values["assetHtml"];
        }
        if (values.hasOwnProperty("assetHtmlLazy") && !settings.htmlLazy) {
            settings.htmlLazy = values["assetHtmlLazy"];
        }
        if (!settings.altTextData) {
            var value = "";
            if (values.hasOwnProperty("assetAltText")) {
                value = values["assetAltText"];
            }
            var choice = "custom";
            if (values.hasOwnProperty("alt-text-choice"))
                choice = values["alt-text-choice"];
            var id = null;
            if (values.hasOwnProperty("alt-text-from-metadata-id")) {
                id = parseInt(values["alt-text-from-metadata-id"]);
            }
            var customValue = "";
            if (values.hasOwnProperty("alt-text-custom")) {
                customValue = values["alt-text-custom"];
            }
            settings.altTextData = { choice: choice, value: value, metadataId: id, customValue: customValue };
        }
        if (!settings.captionData) {
            var value = "";
            if (values.hasOwnProperty("assetCaption")) {
                value = values["assetCaption"];
            }
            var choice = "custom";
            if (values.hasOwnProperty("caption-choice"))
                choice = values["caption-choice"];
            var id = null;
            if (values.hasOwnProperty("caption-from-metadata-id")) {
                id = parseInt(values["caption-from-metadata-id"]);
            }
            var customValue = "";
            if (values.hasOwnProperty("caption-custom")) {
                customValue = values["caption-custom"];
            }
            settings.captionData = { choice: choice, value: value, metadataId: id, customValue: customValue };
        }

        if (values.hasOwnProperty("assetFormatId") && !settings.formatId) {
            settings.formatId = values["assetFormatId"];
        }

        //fix for handling SiteVisions incorrectly conversion from string to javascript object.
        if (values.hasOwnProperty("assetEffects") && !settings.effects) {
            var ae = values["assetEffects"];
            //workaround for SV-27946
            if (ae && typeof (ae) !== "object") {
                ae = JSON.parse(ae);
            }
            settings.effects = ae;
        }
        if (!settings.click) {
            var click = { mode: "none" };
            settings.click = click;
            if (values.hasOwnProperty("clickInteraction")) {
                settings.click.mode = values["clickInteraction"];
            }
            if (values.hasOwnProperty("clickInteractionCustomUrlData")) {
                settings.click.customUrl = values["clickInteractionCustomUrlData"];
            }
        }
        return settings;
    }
    selectAsset(asset: ImageVault.IAsset): Promise<void> {
        return this.infoPanel.selectAsset(asset)
    }


    saveSelection(): IError {
        try {
            this.updateInstanceConfig();
            this.setValue(InstanceConfigUI.AssetConfigKey, JSON.stringify(this.settings.instanceConfig));
        } catch (e) {
            if (e && e.error) return e;
            return { error: e };
        }

    }
    updateInstanceConfig() {
        var assetId = this.settings.instanceConfig.id;
        var altData = this.infoPanel.altTextUI.getData();

        //alt text needs to be at least an empty string to be compliant
        if (!altData.value) {
            altData.value = "";
        }
        var captionData = this.infoPanel.captionUI.getData();
        this.settings.instanceConfig.altTextData = altData;
        this.settings.instanceConfig.captionData = captionData;
        this.settings.instanceConfig.click = this.assetInteractionUI.getClickInteractionData();
        //save format id
        this.settings.instanceConfig.formatId = -1;
        var formatData = this.infoPanel.assetFormatSelector.getSelectedFormatData();
        if (formatData && formatData.id >= 0) {
            this.settings.instanceConfig.formatId = formatData.id;
        }
        var effects = this.infoPanel.assetFormatSelector.getEffectsForTarget();

        //save puff format id
        this.settings.instanceConfig.puff = { formatId: -1 };
        var puffFormatData = this.infoPanel.puffFormatSelector.getSelectedFormatData();
        if (puffFormatData && puffFormatData.id >= 0) {
            this.settings.instanceConfig.puff.formatId = puffFormatData.id;
        }

        var puffEffects = this.infoPanel.puffFormatSelector.getEffectsForTarget();

        //use thumbnail format (works with all except video)
        let formatTemplate = new ImageVault.ThumbnailFormat(effects);
        if (this.infoPanel.isCurrentAssetVideo()) {
            formatTemplate = new ImageVault.WebMediaFormat(effects);
        }
        formatTemplate.MediaFormatOutputType = ImageVault.MediaFormatOutputType.WebSafe;
        let puffFormat = new ImageVault.ThumbnailFormat(puffEffects);
        puffFormat.MediaFormatOutputType = ImageVault.MediaFormatOutputType.WebSafe;

        //no image is selected
        if (!assetId) {
            this.settings.instanceConfig.html = null;
            return;
        }
        let publishIdentifier = "ImageVault_Connect_SiteVision_App";
        //get the site url from the page url
        if (/^((?:\w+:|)\/\/[^\/]+)/.test(this.settings.pageInfo.url)) {
            publishIdentifier = RegExp.$1.toLocaleLowerCase();
        }

        var publishDetails = {
            GroupId: publishIdentifier,
            Text: this.settings.pageInfo.name,
            Url: this.settings.pageInfo.url
        };
        var q: ImageVault.MediaItemQuery = {
            Filter: { Id: [assetId] },
            Populate: {
                MediaFormats: [formatTemplate, puffFormat],
                PublishInfo: { PublishIdentifier: publishIdentifier, PublishDetails: JSON.stringify(publishDetails) }
            },
            MediaUrlBase: this.settings.config.ivUrl
        };
        var maxWidth = this.infoPanel.getCurrentAssetWidth();
        //effects can affect maxWidth
        for (var i = 0; effects && i < effects.length; i++) {
            var e = effects[0];
            if (!e.$type) continue;
            if (e.$type.startsWith(ImageVault.ResizeEffect.typeName)) {
                var re = e as ImageVault.ResizeEffect;
                if (re.Width && re.Width < maxWidth)
                    maxWidth = re.Width;
                continue;
            }
            if (e.$type.startsWith(ImageVault.CropEffect.typeName)) {
                var ce = e as ImageVault.CropEffect;
                if (ce.Width && ce.Width < maxWidth)
                    maxWidth = ce.Width;
                continue;
            }
        }
        //set srcSet limits
        var srcSetLimits = [320, 640, 960, 1280, 1600, 1920];
        var useSrcSet = this.getBooleanValue("SrcSet") && this.infoPanel.isCurrentAssetImage() && maxWidth > srcSetLimits[0];
        if (useSrcSet) {
            //determine limit values
            for (var i = 0; i < srcSetLimits.length; i++) {
                //if limit surpasses maxWidth, remove values larger than the maxValue
                if (srcSetLimits[i] >= maxWidth) {
                    srcSetLimits[i] = maxWidth;
                    srcSetLimits = srcSetLimits.slice(0, i + 1);
                    break;
                }
            }
            for (var i = 0; i < srcSetLimits.length; i++) {
                var resizeEffect = new ImageVault.ResizeEffect(srcSetLimits[i], 0, 0);
                //clone first format
                var format = JSON.parse(JSON.stringify(formatTemplate));
                format.Effects.push(resizeEffect);
                q.Populate.MediaFormats.push(format);
            }
        }

        //if we have an asset that isn't editable (probably a document of some sort), always get the original instead
        if (!this.infoPanel.isCurrentAssetEditable()) {
            q.Populate.MediaFormats = [new ImageVault.OriginalFormat()];
        }
        let mediaItems = ImageVault.MediaService.FindSync(this.settings.client, q);
        if (!mediaItems) {
            throw this.i18n("no-response-was-received");
        }

        if (!mediaItems || !mediaItems.length) {
            throw this.i18n("no-asset-with-id-x-was-found", assetId);
        }
        var mediaItem = mediaItems[0];
        var conversions = mediaItem.MediaConversions;
        if (!conversions || !conversions.length) {
            throw this.i18n("the-requested-conversion-for-asset-x-could-not-be-retrieved", assetId);
        }
        var lazyLoad = this.getBooleanValue("LazyLoad");
        var c = conversions[0] as ImageVault.WebMedia;
        var html = c.Html;
        var htmlLazy = "";
        if (c.ContentType.toLocaleLowerCase().startsWith("image/")) {
            let $e = $("<img/>");
            $e.addClass("sv-noborder"); //added by site vision for all images
            $e.attr("src", c.Url);
            $e.css("width", "100%");
            if (c.Width) {
                // $e.attr("width", c.Width.toString());
                $e.css("max-width", c.Width + "px");
            }
            if (c.Height) {
                // $e.attr("height", c.Height.toString());
                $e.css("max-height", c.Height + "px");
            }
            var srcSet = "";
            if (useSrcSet) {
                for (var i = 0; i < srcSetLimits.length; i++) {
                    if (conversions.length <= i + 2) break;
                    var sWidth = srcSetLimits[i];
                    var media = conversions[i + 2];
                    if (srcSet) srcSet += ", ";
                    srcSet += media.Url + "?w=" + sWidth + " " + sWidth + "w";
                }
                if (srcSet) {
                    $e.attr("srcset", srcSet);
                    $e.attr("sizes", "100vw");
                }
            }
            $e.attr("alt", altData.value);
            html = $e[0].outerHTML;

            if (lazyLoad) {
                $e.attr("data-src", c.Url);
                $e.addClass("sv-lazy-load-img");
                var lazyLoadUrl = "";
                if (c.Width && c.Height) {
                    lazyLoadUrl = "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 " + c.Width + " " + c.Height + "'%3E%3C/svg%3E";
                }
                $e.attr("src", lazyLoadUrl);
                if (srcSet) {
                    $e.attr("data-srcset", srcSet);
                    $e.removeAttr("srcset");
                }
                htmlLazy = $e[0].outerHTML;
            }
        } else {
            //video
            let $e = $(html);
            if ($e[0].tagName.toLocaleLowerCase() === "video") {
                $e.css("width", "100%");
                if (c.Width) {
                    $e.css("max-width", c.Width + "px");
                }
                if (c.Height) {
                    $e.css("max-height", c.Height + "px");
                }
                html = $e[0].outerHTML;
            }
        }
        if (!html)
            throw this.i18n("the-requested-conversion-for-asset-x-has-no-html", assetId);
        this.settings.instanceConfig.html = html;
        this.settings.instanceConfig.htmlLazy = htmlLazy;
        var puff = conversions[1] as ImageVault.WebMedia;
        this.settings.instanceConfig.puff.html = puff.Html;
        return;
    }

}
