import { ISettings } from "./Settings";
import { ConfigUI } from "./ConfigUI";

export class MediaServiceUtilUI extends ConfigUI {
    settings: ISettings;
    private searchMethod: number;

    constructor(settings: ISettings) {
        super(settings);
    }
    init() { }
    async isSearchEngineEnabled(): Promise<boolean> {
        if (!this.searchMethod) {
            try {
                var r = await ImageVault.AssetService.info(this.settings.client);
                this.searchMethod = (r.searchEnabled ? 1 : 2);
            } catch (e) {
                this.logResult("ERROR", "Failed to lookup assets/info " + e);
                this.searchMethod = 2;
            }
        }
        return this.searchMethod === 1;
    }
    async getAsset(id: number): Promise<ImageVault.IAssetResponse> {
        if (await this.isSearchEngineEnabled()) {
            return await ImageVault.AssetService.get(this.settings.client, { $filter: "id eq '" + id + "'" });
        }
        return await this.getUsingMediaService(id);
    }
    private async getUsingMediaService(id: number): Promise<ImageVault.IAssetResponse> {
        var q: ImageVault.MediaItemQuery = {
            Filter: {
                Id: [id]
            },
            Populate: {
                MediaFormats: [
                    {},//original
                ],
                Metadata: [{ Filter: { IsNull: false } }],
                Agreements: true,
                Categories: true
            },
            MediaUrlBase: null//this.settings.config.ivUrl + "apiv2/mediastreamservice"
        };

        var r = await ImageVault.MediaService.Find(this.settings.client, q);
        var re: ImageVault.IAssetResponse = {
            "@odata.count": 0,
            value: []
        };
        //no data, return empty result
        if (!r) {
            return re;
        }

        re.value = this.convertMediaItemsToIAssets(r);
        re["@odata.count"] = re.value.length;
        return re;
    }
    async search(request: ImageVault.IAssetRequest, categoryId: number[]): Promise<ImageVault.IAssetResponse> {
        if (await this.isSearchEngineEnabled()) {
            return await ImageVault.AssetService.get(this.settings.client, request);
        }
        return await this.searchUsingMediaService(request, categoryId);
    }
    private async searchUsingMediaService(request: ImageVault.IAssetRequest, categoryId: number[]): Promise<ImageVault.IAssetResponse> {
        var q: ImageVault.MediaItemQuery = {
            Filter: {
                SearchString: request.$search,
                Skip: request.$skip,
                Take: request.$top + 1,
                SortOrder: ImageVault.MediaItemSortOrders.DateAddedDescending,
                CategoryId: categoryId
            },
            Populate: {
                MediaFormats: [
                    {},//original
                    new ImageVault.ThumbnailFormat([new ImageVault.ResizeEffect(200, 200, 0)])
                ],
                Metadata: [{ Filter: { IsNull: false } }],
                Agreements: true,
                Categories: true
            },
            MediaUrlBase: null//this.settings.config.ivUrl + "apiv2/mediastreamservice"
        };
        if (request.$orderby) {
            switch (request.$orderby) {
                case "name":
                    q.Filter.SortOrder = ImageVault.MediaItemSortOrders.Name;
                    break;
                case "name desc":
                    q.Filter.SortOrder = ImageVault.MediaItemSortOrders.NameDescending;
                    break;
                case "modified":
                    q.Filter.SortOrder = ImageVault.MediaItemSortOrders.DateAdded;
                    break;
                case "modified desc":
                    q.Filter.SortOrder = ImageVault.MediaItemSortOrders.DateAddedDescending;
                    break;
                default:
                    this.logResult("WARNING", "Unknown sort order " + request.$orderby);
                    break;
            }
        }
        var r = await ImageVault.MediaService.Find(this.settings.client, q);
        var re: ImageVault.IAssetResponse = {
            "@odata.count": request.$skip || 0,
            value: []
        };
        //no data, return empty result
        if (!r) {
            return re;
        }
        //simulate paging
        var moreResultExists = r.length > request.$top;
        if (moreResultExists) {
            r = r.slice(0, request.$top);
        }
        re["@odata.count"] += r.length + (moreResultExists ? 1 : 0);
        re.value = this.convertMediaItemsToIAssets(r);
        return re;
    }
    private convertMediaItemsToIAssets(r: ImageVault.MediaItem[]): ImageVault.IAsset[] {
        return r.map((mi) => {
            var asset: ImageVault.IAsset = {};
            asset.id = mi.Id.toString();
            asset.name = mi.Name;
            asset.vaultId = mi.VaultId;
            asset.modified = mi.DateAdded;
            asset.modifiedBy = mi.AddedBy;
            asset.symbols = [];
            asset.metadata = [];
            var symbols: { [key: string]: ImageVault.AssetSymbolInfo[] } = {};
            if (mi.Metadata && mi.Metadata.length) {
                asset.metadata = mi.Metadata.map((md) => {
                    if (!md || md.IsNull) return null;
                    var am: ImageVault.AssetMetadata = {
                        definitionId: md.MetadataDefinitionId,
                        value: (md.Value || "").toString()
                    };
                    var symbolKey = md.SymbolKey;
                    if (md.$type.startsWith("ImageVault.Common.Data.MetadataBoolean")) {
                        am.booleanValue = md.Value;
                        if (!am.booleanValue) {
                            symbolKey = null;
                        }
                    } else if (md.$type.startsWith("ImageVault.Common.Data.MetadataDecimal")) {
                        am.doubleValue = md.Value;
                    } else if (md.$type.startsWith("ImageVault.Common.Data.MetadataDateTime")) {
                        am.dateValue = am.value;
                    } else if (md.$type.startsWith("ImageVault.Common.Data.MetadataInteger")) {
                        am.integerValue = md.Value
                    }
                    if (symbolKey) {
                        var list = symbols[symbolKey];
                        if (!list) {
                            list = [];
                            symbols[symbolKey] = list;
                        }
                        var info: ImageVault.AssetSymbolInfo = {
                            name: md.Name,
                            value: md.Value
                        };
                        list.push(info);
                    }
                    return am;
                }).filter((a) => a != null);
            }
            Object.keys(symbols).map((k) => {
                var as: ImageVault.AssetSymbol = {
                    symbol: k,
                    info: symbols[k]
                };
                asset.symbols.push(as);
            });
            var agreementSymbol = this.getAgreementSymbol(mi.Agreements);
            if (agreementSymbol)
                asset.symbols.push(agreementSymbol);

            asset.conversions = [];
            if (mi.MediaConversions && mi.MediaConversions.length) {
                var original = mi.MediaConversions[0];
                mi.MediaConversions = mi.MediaConversions.slice(1);
                asset.conversions = mi.MediaConversions.map((mc) => {
                    var wm = mc as ImageVault.WebMedia;
                    var ac: ImageVault.AssetConversion = {
                        formatId: mc.MediaFormatId,
                        contentType: mc.ContentType,
                        height: wm.Height,
                        width: wm.Width,
                        url: mc.Url,
                        mediaInformation: null
                    }
                    return ac;
                });
                if (original) {
                    asset.contentType = original.ContentType
                }
            }
            asset.categories = [];
            if (mi.Categories && mi.Categories.length) {
                asset.categories = mi.Categories.map((c) => {
                    return {
                        id: c.Id,
                        name: c.Name
                    };
                });
            }
            return asset;
        });

    }
    private getAgreementSymbol(agreements: ImageVault.AgreementListData[]): ImageVault.AssetSymbol {
        if (!agreements || !agreements.length) return null;
        //expired
        var now = new Date();
        if (this.hasAgreementPassingValidDate(agreements, now)) {
            return { symbol: "expired", info: [{ name: this.i18n("has_expired_agreements") }] }
        }
        var soonExpired = new Date();
        soonExpired.setMonth(soonExpired.getMonth() + 3);
        if (this.hasAgreementPassingValidDate(agreements, soonExpired)) {
            return { symbol: "soonexpired", info: [{ name: this.i18n("has_soon_expired_agreements") }] }
        }
        return { symbol: "valid", info: [{ name: this.i18n("all_agreements_are_valid") }] }
    }
    private hasAgreementPassingValidDate(agreements: ImageVault.AgreementListData[], validDate: Date): boolean {
        var expired = agreements.filter((a) => {
            var date = new Date(a.ValidToDate);
            return date < validDate;
        });
        return !!expired.length;
    }

}