import { ConfigUI } from "./ConfigUI";
import { IInstanceBroker } from "./InstanceConfigUI";
import { IThumbnailSelectorData } from "./Settings";

export interface IThumbSelectorProps {
    prefix: string;
    targetSize: number;
    thumbEdited?: () => void;
    data: IThumbnailSelectorData;
}
export class ThumbSelectorUI extends ConfigUI {
    props: IThumbSelectorProps;
    $formatWidth: JQuery;
    $formatHeight: JQuery;
    $formatSelect: JQuery;
    $formatAspectRatio: JQuery;
    $thumbnailLoader: JQuery;
    $thumbnailLoaderSpinner: JQuery;
    $thumbnailLoaderErrorMessage: JQuery;
    formatSelectorEnabled: boolean;
    $assetEditor: JQuery;
    $thumbImg: JQuery;
    broker: IInstanceBroker;

    constructor(broker: IInstanceBroker, props: IThumbSelectorProps) {
        super(broker.settings);
        this.broker = broker;
        this.props = props;
        this.formatSelectorEnabled = true;
    }

    init() {
        var self = this;
        this.$formatSelect = this.getInput(this.props.prefix + "-formatId");
        this.$formatWidth = this.getInput(this.props.prefix + "-formatWidth");
        this.$formatHeight = this.getInput(this.props.prefix + "-formatHeight");
        this.$formatAspectRatio = this.getInput(this.props.prefix + "-formatAspectRatio");
        this.$thumbnailLoader = $("#" + this.props.prefix + "-thumb-loader");
        this.$thumbnailLoaderSpinner = $(".spinner", this.$thumbnailLoader);
        this.$thumbnailLoaderErrorMessage = $(".alert", this.$thumbnailLoader);
        this.$thumbImg = $("img#" + this.props.prefix + "-thumb");
        this.$assetEditor = $("#asset-editor");

        this.$formatSelect.change(() => {

            var selectedFormat = self.getSelectedFormatDataInner();
            var width, height;
            if (selectedFormat.width)
                width = selectedFormat.width.toString();
            if (selectedFormat.height)
                height = selectedFormat.height.toString();
            if (width) width += " px";
            if (height) height += " px";

            self.$formatWidth.val(width);
            self.$formatHeight.val(height);
            if (selectedFormat.resizeMode) {
                self.$formatAspectRatio.val(selectedFormat.aspectRatioText);
            } else {
                self.$formatAspectRatio.val("");
            }
            self.updateThumbnail();
        });
    }
    getEffectsForTarget(): ImageVault.IEffect[] {
        var effects = this.settings.instanceConfig.effects || [];
        effects = effects.slice();

        var format = this.getSelectedFormatData();

        if (format && (format.width || format.height)) {
            var resizeEffect = new ImageVault.ResizeEffect(0, 0, 0);
            resizeEffect.Width = format.width;
            resizeEffect.Height = format.height;
            resizeEffect.ResizeMode = format.resizeMode;
            //do not add effect if last effect is already the one suggested
            var resizeAlreadyAdded = false;
            var lastEffect = null;
            if (effects.length > 0) {
                lastEffect = effects[effects.length - 1];
                if (lastEffect.$type === resizeEffect.$type) {
                    var lastResize = lastEffect as ImageVault.ResizeEffect;
                    if (lastResize.Width == resizeEffect.Width && lastResize.Height == resizeEffect.Height && lastResize.ResizeMode == resizeEffect.ResizeMode)
                        resizeAlreadyAdded = true;
                }
            }
            if (!resizeAlreadyAdded)
                effects.push(resizeEffect);
        }
        return effects;
    }

    getSelectedFormatData(): IFormatData {
        //return none if format dialog is hidden
        if (!this.formatSelectorEnabled)
            return null;
        return this.getSelectedFormatDataInner();

    }
    private getSelectedFormatDataInner(): IFormatData {
        var $option = this.$formatSelect.find(":selected")
        var width = 0;
        var id = this.getIntValue(this.$formatSelect);
        if ($option.data("width"))
            width = Math.round(parseFloat($option.data("width")));
        var height = 0;
        if ($option.data("height"))
            height = Math.round(parseFloat($option.data("height")));
        var resizeMode = parseInt($option.data("resizeMode"));
        var aspectRatioText = $option.data("aspectRatioText");
        var formatData = { id: id, width: width, height: height, resizeMode: resizeMode, aspectRatioText: aspectRatioText }
        //this.logResult("INFO", JSON.stringify(formatData));
        return formatData;
    }
    enable(enable: boolean) {
        this.formatSelectorEnabled = enable;
    }
    async updateFormats() {
        var self = this;
        //query all user formats
        var q = {
            Filter: {
                Format: {
                    SystemFormat: false
                }
            }
        };
        var suggestedFormatId = 0;
        var closestFormatDiff = -1;

        try {
            var formats = await ImageVault.MediaFormatService.Find(self.settings.client, q);
            self.$formatSelect.empty();
            self.$formatSelect.append('<option value="0">Original</option>');
            for (var i = 0; i < formats.length; i++) {
                var f = formats[i] as ImageVault.WebMediaFormat;
                let width: number;
                let height: number;
                let resizeMode: number = 0;
                if (f.Width) width = f.Width;
                if (f.Height) height = f.Height;
                if (f.AspectRatio) {
                    if (width) {
                        height = width / f.AspectRatio;
                        resizeMode = 1;
                    } else if (height) {
                        width = height * f.AspectRatio;
                        resizeMode = 1;
                    }
                }
                var aspectRatioText = "";
                if (resizeMode) {
                    var aspectRatio = width / height;
                    aspectRatioText = this.calculateAspectRatioDisplayValue(aspectRatio);
                }
                //Calculate which format is closest to the suggested format size
                //assume width & height are equal if only one is supplied
                var fw = width || height;
                var fh = height || width;
                //format is only interesting if we specify width and height
                if (fw && fh) {
                    var formatDiff = Math.abs(this.props.targetSize - fw) + Math.abs(this.props.targetSize - fh);
                    //if the new format is closer to the target size, use that instead
                    if (closestFormatDiff < 0 || formatDiff < closestFormatDiff) {
                        closestFormatDiff = formatDiff;
                        suggestedFormatId = f.Id;
                    }
                }
                var $o = $("<option></option>")
                    .attr("value", f.Id)
                    .data("width", (width ? width.toString() : ""))
                    .data("height", (height ? height.toString() : ""))
                    .data("resizeMode", resizeMode)
                    .data("aspectRatioText", aspectRatioText)
                    .text(f.Name);
                self.$formatSelect.append($o);
            }
            //if no current format is chosen, use the suggested format
            var currentFormatId = self.props.data.formatId;
            if (currentFormatId === undefined || currentFormatId === null || currentFormatId < 0) {
                currentFormatId = suggestedFormatId;
            }
            self.$formatSelect.val(currentFormatId);
            self.$formatSelect.trigger("change");
        } catch (r) {
            self.error(self.i18n("failed-to-update-formats"), r);
        }
    }
    calculateAspectRatioDisplayValue(ar: number): string {
        if (ar <= 0)
            return "";

        var r = ar;
        //max denominator to try
        const maxDenominator = 9;
        //what diff is acceptable?
        const okDiff = 0.0001;
        //find the closest whole number fraction
        for (var d = 1; d <= maxDenominator; d++) {
            var n = Math.trunc(Math.round(d * r));
            var r2 = 1.0 * n / d;
            var diff = Math.abs(r - r2);
            //if diff is not close enough, try again
            if (diff > okDiff) {
                continue;
            }
            //good fraction
            return n + ":" + d;
        }
        return r.toString();
    }
    showThumbnailLoader() {
        this.$thumbnailLoaderSpinner.show();
        this.$thumbnailLoaderErrorMessage.hide();
        this.$thumbnailLoader.show();
    }
    showThumbnailLoaderErrorMessage(message: string) {
        this.$thumbnailLoaderErrorMessage.text(message);
        this.$thumbnailLoaderSpinner.hide();
        this.$thumbnailLoaderErrorMessage.show();
        this.$thumbnailLoader.show();
    }
    hideThumbnailLoader() {
        this.$thumbnailLoader.hide();
    }
    updateThumbnail() {
        var id = this.settings.instanceConfig.id;
        this.$thumbImg.hide();
        this.$thumbImg.attr("src", "")
        if (!id) return;
        this.showThumbnailLoader();

        var self = this;
        //use target effects instead. By that we get the target image on preview and it's already converted when loading the page.
        var effects = self.getEffectsForTarget();
        var format = new ImageVault.ThumbnailFormat();
        format.MediaFormatOutputType = ImageVault.MediaFormatOutputType.WebSafe;
        format.Effects = effects;
        let query: ImageVault.MediaItemQuery = {
            Filter: { Id: [id] },
            Populate: { MediaFormats: [format] },
            MediaUrlBase: self.settings.config.ivUrl + "apiv2/mediastreamservice"
        };
        var accessToken = self.settings.client.getCachedAccessToken();
        var imageQs = "?ivauthorization=" + encodeURIComponent(accessToken)

        self.getMedia(query).then((media) => {
            self.$thumbImg.attr("src", media.Url + imageQs).attr("width", media.Width).attr("height", media.Height);
            self.$thumbImg.show();
            self.$thumbImg.on("load", () => { self.hideThumbnailLoader(); });
        }).catch((e) => {
            // self.error("", e);
            self.showThumbnailLoaderErrorMessage(e);
        });

    }
    editAsset() {
        var self = this;

        var $iframe = $("#editMedia");
        $iframe.on("load", function () {
            self.$assetEditor.show();
            self.logResult("DEBUG", "Iframe loaded: " + $iframe.attr("src"));
            $iframe.unbind("load");
            $iframe.unbind("keydown");
            var win = ($iframe.get(0) as any).contentWindow;
            self.initEditor(win);
        });
        $iframe.attr("src", self.settings.resourcePath + "imagevault-client-script/editor.html?v=4&timestamp=" + (new Date()).getTime());
    }
    initEditor(win) {
        var self = this;
        var id = this.settings.instanceConfig.id;
        if (!id) {
            console.log("Cannot edit asset if no one is selected");
            return;
        }
        var selectedFormat = this.getSelectedFormatData();
        var keepAspectRatio = false;
        //we need to use thumbnail format to be able to edit document types (pdf/eps) since WebMediaFormats is not supported
        var format: ImageVault.MediaFormatBase = new ImageVault.ThumbnailFormat();
        if (this.broker.infoPanel.isCurrentAssetVideo()) {
            format = new ImageVault.WebMediaFormat()
        }
        format.MediaFormatOutputType = ImageVault.MediaFormatOutputType.WebSafe;

        self.getMedia({
            Filter: { Id: [id] },
            Populate: {
                MediaFormats: [format]
            },
            MediaUrlBase: self.settings.config.ivUrl + "apiv2/mediastreamservice"
        }).then((media) => {
            if (!media) {
                throw "Failed to retrieve media for asset id " + id + " and format " + JSON.stringify(format);
            }
            var effectArray = self.settings.instanceConfig.effects;
            var effects = JSON.stringify(effectArray);
            var params = {
                id: id,
                editorSettings: { width: win.innerWidth, height: win.innerHeight },
                coreSettings: {
                    client: self.settings.client,
                    mediaUrlBase: self.settings.config.ivUrl + "apiv2/mediastreamservice",
                    useMediaAuthenticationParameter: true,
                },
                languageSettings: {
                    editorWidthInputLabel: self.i18n("width"),
                    editorHeightInputLabel: self.i18n("height"),
                    editorOkButtonText: self.i18n("ok"),
                    editorCancelButtonText: self.i18n("cancel"),
                    editorWarningIconText: self.i18n("warning-image-is-scaled-to-fit-the-area"),
                    editorReloadButtonText: self.i18n("reload"),
                    editorLoadErrorText: self.i18n("editor-load-error"),
                    title: self.i18n("editor-title")
                },
                currentEffects: effects,
                //original image format
                originalFormat: {
                    width: media.Width,
                    height: media.Height,
                    url: media.Url,
                    contentType: media.ContentType
                },
                originalFormatContentType: media.ContentType,
                originalFormatUrl: media.Url,
                keepAspectRatio: keepAspectRatio,
                nodeInfo: null,

                targetFormat: {
                    width: (selectedFormat ? selectedFormat.width : 0),
                    height: (selectedFormat ? selectedFormat.height : 0),
                    resizeMode: (selectedFormat ? selectedFormat.resizeMode : 0)
                },
                //return an array of effects that should be applied to the media
                callback: function (effects) {
                    //if cancel is pressed, effects is null
                    if (effects) {
                        if (typeof (effects) === "string") {
                            effects = JSON.parse(effects);
                        }
                        self.settings.instanceConfig.effects = effects;
                        if (effects && effects.length > 0) {
                            self.logResult("INFO", effects);
                        }
                        self.updateThumbnail();
                        if (self.props.thumbEdited) {
                            self.props.thumbEdited();
                        }
                    }
                    self.$assetEditor.hide();
                }
            };
            win.ImageVault.Editor.openEditorInstance(params);
        }).catch((e) => {
            self.error(e);
            self.$assetEditor.hide();
        });
    }
    clearThumb() {
        this.$thumbImg.hide();
        this.$thumbImg.attr("src", "");
    }
}

interface IFormatData {
    id: number;
    width: number;
    height: number;
    resizeMode: number;
    aspectRatioText: string;
}